
	/* definitions for the "multipht" program */	

	/*
	 * added new fields "xshift" and "yshift" to the s_image structure
	 *   so that we can un-shift stars after matching.
	 * MWR 12/15/2002
	 * 
	 * added field "num_good_star" to the s_image structure;
	 *   it keeps track of the number of stars in each image
	 *   which will be included in the ensemble output.
	 * also, increased MAX_STAR from 1000 to 10,000 
	 *   and MAX_IMAGE from 1000 to 3000
	 * MWR 11/20/2004
	 */



#define MAX_STAR  10000	/* max number stars total we can have */
#define MAX_IMAGE 3000	/* max number of images we can have */
#define STRLEN    40		/* max length of star names and filters */
#define LINELEN   256
#define LINELEN   256		/* max length of data lines */
#define NODATA    99.0		/* value assigned to a variable w/ no data */
#define MAXMATCH  40        /* Use the brightest MAXMATCH stars to determine */
                            /*    offset between images */
#define MAX_COL   20       /* max number of columns in input files */

	/* parameters used to determine if stars match or not */
#define BIG       1.0e8	   /* a number >> max value of position coords */
#define DEF_RADIUS   5.0	/* default distance (in coord units) within */
                           /*   which two stars count as 'matched' */
#define MAX_COORD  4000    /* (x,y) coords must be less than this */

	/* some flags which tell the program its function */
#define PRINT     1			
#define AUTO      2			/* don't ask for input from user */

	/* this character denotes a comment in data files */
#define COMMENT_CHAR  '#'

   /*
    * This is a preprocessor macro, which acts like a function.  The user
    * calls it with one argument, a condition which should evaluate to
    * 0 or 1.  If it evaluates to 1, then nothing happens; but if it
    * evaluates to 0, then the program prints an error message, giving
    * location of the error, and halts execution with an error code.
    * 
    * Thus, one typically uses it to make a 'sanity check', such as
    * making sure that a pointer (which really, really should have
    * a valid value) isn't NULL:
    *
    *     fp = fopen("file", "r");
    *        ...
    *     shAssert(fp != NULL);
    *     fgets(line, LINELEN, fp);
    *        ...
    */
#define shAssert(x) if((x)!=1){fprintf(stderr,"assertion fails in file %s, line %d\n",__FILE__,__LINE__);exit(1);}
 
typedef int (*PFI)();

struct s_image {
	int index;				/* identifying index used in output routines */
	char name[STRLEN];
	double exptime;
	double jd;           /* some time value, not necessarily Julian Date */
	int num_star;			/* number of stars in the image */
	struct s_star *stars;	/* array of stars in this image */
	int num_good_star;   /* # stars which will be included in ensemble */
	int num_match;			/* # stars that match template image */
	double xshift;       /* x-shift to match fiducial image */
	double yshift;       /* y-shift to match fiducial image */
	int include;			/* if 1 include stars from this image; if 0 don't */
};

struct s_star {
	int image_id;				/* ID number on this star's image */
	int id;						/* ID number in set of all images' stars */
	struct s_image *image;
	double xc;
	double yc;
	double jd;              /* Julian Date, or any time value */
	double mag;             /* input instrumental magnitude */
	double err;             /* uncertainty in instrumental magnitude */
	int w3;						/* yes/no weight for this star, this image */
	double w4;					/* weight calculated from signal/noise */
	int saturated;				/* yes/no is this star saturated? */
};

	/* some structures used mostly for hashing stars, but also
	   for going through the total group of stars by ID number */

struct s_starp {
	struct s_star *star;
	struct s_starp *next;
};

struct s_hash {
	double xc, yc;
	int id;
	int num;					   /* number of stars with this ID */
	struct s_starp *starp;  /* a linked list of stars with this ID */
	struct s_hash *next;		/* linked list of other stars with similar */
	                    		/*   hash values, but different positions */
	int include;				/* yes/no include star w/this ID in solution */
};

struct s_hashp {
	struct s_hash *hash;
	struct s_hashp *next;
};


	/* these are the heads of some important arrays/lists
	             image_arr: an array of all the images
	             star_list: a list of all the instances of the same star
	 */

struct s_image *image_arr;
struct s_hashp *star_list;

	/* some global variables */

int debug_flag;
int num_images;			/* total number of images */
int num_ids;			   /* total number of unique stars */
double RADIUS;                 /* radius used to match stars */
double RAD2;                   /* square of matching radius */

	/* some global functions */
void print_star_hist(void);
void print_image_hist(void);
int assign_ids(void);
void write_output(char *file, int flags);
int do_match(char *template);
void do_weights(int flags);
